@@ -1,4 +1,4 @@
-#Copyright ReportLab Europe Ltd. 2000-2016
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/platypus/tables.py
 __version__='3.3.0'
@@ -204,7 +204,7 @@ class Table(Flowable):
     def __init__(self, data, colWidths=None, rowHeights=None, style=None,
                 repeatRows=0, repeatCols=0, splitByRow=1, emptyTableAction=None, ident=None,
                 hAlign=None,vAlign=None, normalizedData=0, cellStyles=None, rowSplitRange=None,
-                spaceBefore=None,spaceAfter=None):
+                spaceBefore=None,spaceAfter=None, longTableOptimize=None):
         self.ident = ident
         self.hAlign = hAlign or 'CENTER'
         self.vAlign = vAlign or 'MIDDLE'
@@ -218,7 +218,8 @@ class Table(Flowable):
         elif colWidths and _seqCW: ncols = len(colWidths)
         else: ncols = 0
         if not emptyTableAction: emptyTableAction = rl_config.emptyTableAction
-        self._longTableOptimize = getattr(self,'_longTableOptimize',rl_config.longTableOptimize)
+        self._longTableOptimize = (getattr(self,'_longTableOptimize',rl_config.longTableOptimize)
+                                    if longTableOptimize is None else longTableOptimize)
         if not (nrows and ncols):
             if emptyTableAction=='error':
                 raise ValueError("%s must have at least a row and column" % self.identity())
@@ -504,7 +505,7 @@ class Table(Flowable):
                 if isinstance(w,(float,int)): return w
             except AttributeError:
                 pass
-        if v is None: 
+        if v is None:
             return 0
         else:
             try:
@@ -583,28 +584,34 @@ class Table(Flowable):
                 H[i] = h
                 # we can stop if we have filled up all available room
                 if longTable:
-                    hmax = i
-                    height = sum(H[:i])
+                    hmax = i+1      #we computed H[i] so known len == i+1
+                    height = sum(H[:hmax])
                     if height > availHeight:
-                        #we can terminate if all spans are complete in H[:i]
+                        #we can terminate if all spans are complete in H[:hmax]
                         if spanCons:
-                            msr = max([x[1] for x in spanCons.keys()])  #RS=[endrowspan,.....]
-                            if hmax>=msr:
+                            msr = max(x[1] for x in spanCons.keys())    #RS=[endrowspan,.....]
+                            if hmax>msr:
                                 break
             if None not in H: hmax = lim
 
             if spanCons:
                 try:
-                    spanFixDim(H0,H,spanCons,lim=hmax)
+                    spanFixDim(H0,H,spanCons)
                 except:
                     annotateException('\nspanning problem in %s hmax=%s lim=%s avail=%s x %s\nH0=%r H=%r\nspanCons=%r' % (self.identity(),hmax,lim,availWidth,availHeight,H0,H,spanCons))
 
-        height = self._height = sum(H[:hmax])
-        self._rowpositions = [height]    # index 0 is actually topline; we skip when processing cells
-        for h in H[:hmax]:
-            height -= h
-            self._rowpositions.append(height)
-        assert abs(height)<1e-8, '!!!!!%s\ninternal height error height=%r hmax=%d Sum(H[:%d])=%r\nH=%r\nrowPositions=%r' % (self.identity(),height,hmax,hmax,self._height,H[:hmax],self._rowpositions)
+        #iterate backwards through the heights to get rowpositions in reversed order
+        self._rowpositions = j = []
+        height = c = 0
+        for i in xrange(hmax-1,-1,-1):
+            j.append(height)
+            y = H[i] - c
+            t = height + y
+            c = (t - height) - y
+            height = t
+        j.append(height)
+        self._height = height
+        j.reverse()     #reverse the reversed list of row positions
         self._hmax = hmax
 
     def _calc(self, availWidth, availHeight):
@@ -642,7 +649,7 @@ class Table(Flowable):
 
     def _culprit(self):
         """Return a string describing the tallest element.
-        
+
         Usually this is what causes tables to fail to split.  Currently
         tables are the only items to have a '_culprit' method. Doctemplate
         checks for it.
@@ -652,10 +659,10 @@ class Table(Flowable):
         rowNum = rh.index(tallest)
         #rowNum of limited interest as usually it's a split one
         #and we see row #1.  Text might be a nice addition.
-        
+
         return 'tallest cell %0.1f points' % tallest
-        
-        
+
+
 
     def _hasVariWidthElements(self, upToRow=None):
         """Check for flowables in table cells and warn up front.
@@ -728,12 +735,7 @@ class Table(Flowable):
                 for rowNo in xrange(self._nrows):
                     value = self._cellvalues[rowNo][colNo]
                     style = self._cellStyles[rowNo][colNo]
-                    pad = style.leftPadding+style.rightPadding
-                    new = elementWidth(value,style)
-                    if new:
-                        new += pad
-                    else:
-                        new = pad
+                    new = elementWidth(value,style) or 0
                     new += style.leftPadding+style.rightPadding
                     final = max(final, new)
                     siz = siz and self._canGetWidth(value) # irrelevant now?
@@ -960,7 +962,7 @@ class Table(Flowable):
         Based on self._spanRanges, which is already known,
         and the widths which were given or previously calculated,
         self._spanRects shows the real coords for drawing:
-            
+
             (col, row) -> (x, y, width, height)
 
         for each cell.  Any cell which 'does not exist' as another
@@ -1260,11 +1262,17 @@ class Table(Flowable):
         #we're going to split into two superRows
         ident = self.ident
         if ident: ident = IdentStr(ident)
-        R0 = self.__class__( data[:n], colWidths=self._colWidths, rowHeights=self._argH[:n],
+        lto = self._longTableOptimize
+        if lto:
+            splitH = self._rowHeights
+        else:
+            splitH = self._argH
+        R0 = self.__class__( data[:n], colWidths=self._colWidths, rowHeights=splitH[:n],
                 repeatRows=repeatRows, repeatCols=repeatCols,
                 splitByRow=splitByRow, normalizedData=1, cellStyles=self._cellStyles[:n],
                 ident=ident,
-                spaceBefore=getattr(self,'spaceBefore',None))
+                spaceBefore=getattr(self,'spaceBefore',None),
+                longTableOptimize=lto)
 
         nrows = self._nrows
         ncols = self._ncols
@@ -1324,22 +1332,23 @@ class Table(Flowable):
             if isinstance(repeatRows,int):
                 iRows = data[:repeatRows]
                 nRepeatRows = repeatRows
-                iRowH = self._argH[:repeatRows]
+                iRowH = splitH[:repeatRows]
                 iCS = self._cellStyles[:repeatRows]
             else:
                 #we have a list of repeated rows eg (1,3)
                 repeatRows = list(sorted(repeatRows))
                 iRows = [data[i] for i in repeatRows]
                 nRepeatRows = len(repeatRows)
-                iRowH = [self._argH[i] for i in repeatRows]
+                iRowH = [splitH[i] for i in repeatRows]
                 iCS = [self._cellStyles[i] for i in repeatRows]
             R1 = self.__class__(iRows+data[n:],colWidths=self._colWidths,
-                    rowHeights=iRowH+self._argH[n:],
+                    rowHeights=iRowH+splitH[n:],
                     repeatRows=nRepeatRows, repeatCols=repeatCols,
                     splitByRow=splitByRow, normalizedData=1,
                     cellStyles=iCS+self._cellStyles[n:],
                     ident=ident,
                     spaceAfter=getattr(self,'spaceAfter',None),
+                    longTableOptimize=lto,
                     )
             R1._cr_1_1(n,nRepeatRows,A)
             R1._cr_1_1(n,nRepeatRows,self._bkgrndcmds)
@@ -1352,6 +1361,7 @@ class Table(Flowable):
                     splitByRow=splitByRow, normalizedData=1, cellStyles=self._cellStyles[n:],
                     ident=ident,
                     spaceAfter=getattr(self,'spaceAfter',None),
+                    longTableOptimize=lto,
                     )
             R1._cr_1_0(n,A)
             R1._cr_1_0(n,self._bkgrndcmds)
@@ -1475,15 +1485,40 @@ class Table(Flowable):
                         canv.rect(x0, y0, w, h, stroke=0,fill=1)
                     x0 = x0 +w
             else:   #cmd=='BACKGROUND'
-                color = colors.toColorOrNone(arg)
-                if color:
+                if arg and isinstance(arg,(list,tuple)) and arg[0] in ('VERTICAL','HORIZONTAL'):
+                    #
+                    # Arg is a list, assume we are going for a gradient fill
+                    # where we expect a containing a direction for the gradient
+                    # and the starting an final gradient colors. For example:
+                    # ['HORIZONTAL', colors.white, colors.grey]   or
+                    # ['VERTICAL', colors.red, colors.blue]
+                    #
+                    canv.saveState()
+
                     if ec==sc and er==sr and spanRects:
                         xywh = spanRects.get((sc,sr))
                         if xywh:
                             #it's a single cell
                             x0, y0, w, h = xywh
-                    canv.setFillColor(color)
-                    canv.rect(x0, y0, w, h, stroke=0,fill=1)
+                    p = canv.beginPath()
+                    p.rect(x0, y0, w, h)
+                    canv.clipPath(p, stroke=0)
+                    direction=arg.pop(0)
+                    if direction=="HORIZONTAL":
+                        canv.linearGradient(x0,y0,x0+w,y0,arg,extend=False)
+                    else:   #VERTICAL
+                        canv.linearGradient(x0,y0,x0,y0+h,arg,extend=False)
+                    canv.restoreState()
+                else:
+                    color = colors.toColorOrNone(arg)
+                    if color:
+                        if ec==sc and er==sr and spanRects:
+                            xywh = spanRects.get((sc,sr))
+                            if xywh:
+                                #it's a single cell
+                                x0, y0, w, h = xywh
+                        canv.setFillColor(color)
+                        canv.rect(x0, y0, w, h, stroke=0,fill=1)
 
     def _drawCell(self, cellval, cellstyle, pos, size):
         colpos, rowpos = pos
